import thad_roberts_model.measurement
import thad_roberts_model.particles_masses
import thad_roberts_model.thad_roberts_model
from thad_roberts_model.measurement import *
from thad_roberts_model.particles_masses import *
from thad_roberts_model.thad_roberts_model import *


from .measurement import *
from .particles_masses import *

class ThadRobertsModel(PartliclesMasses):
    RS_CONST = v("1.45136923488338105028396848589202744949303228")
    PI_CONST = v(math.pi)
    EQ_CONST = RS_CONST + v("3.0") + PI_CONST
    
    RSSVE_1 = v("0.0854245431533310")
    RSSVE_2 = v("3.667567534854999")
    
    RSSVE_5 = v("0.093474041000289951749648")
    RSSVE_6 = v("3.526755530609707752174473")
    
    RSSVE_CORRECTION = v("1") + ((RSSVE_1**v("2")) * (RSSVE_5**v("2")))/((RSSVE_2**v("2")) * (RSSVE_6**v("2")))
    
    def __init__(self):
        self.equations = {}
    
    def verify(self):
        self.equations[('HIGGS', 'Z_BOSON', 'W_BOSON')] = ((v("3")**v("2"))*(self.HIGGS - self.Z_BOSON))/((((v("2")**v("-1")))*self.W_BOSON)*self.RSSVE_CORRECTION)
        self.equations[('NEUTRON', 'PROTON', 'ELECTRON')] = ((v("3")**v("1"))*(self.NEUTRON - self.PROTON))/((((v("2")**v("0")))*self.ELECTRON)*self.RSSVE_CORRECTION)
        self.equations[('DOWN_QUARK', 'UP_QUARK', 'ELECTRON')] = ((v("3")**v("1"))*(self.DOWN_QUARK - self.UP_QUARK))/((((v("2")**v("1")))*self.ELECTRON)*self.RSSVE_CORRECTION)
        self.equations[('CHARM_QUARK', 'STRANGE_QUARK', 'MUON')] = ((v("3")**v("1"))*(self.CHARM_QUARK - v("2")*self.STRANGE_QUARK))/((((v("2")**v("2")))*self.MUON)*self.RSSVE_CORRECTION)
        self.equations[('TOP_QUARK', 'TAU', 'BOTTOM_QUARK')] = ((v("3")**v("1"))*(self.TOP_QUARK - v("2")*self.TAU))/((((v("2")**v("4")))*self.BOTTOM_QUARK)*self.RSSVE_CORRECTION)
        
        for eq_key, value in self.equations.items():
            print("RS+3+PI calculation for equation", eq_key, ":")
            print("[   ", self.RS_CONST, "= RS+3+PI", "   ]")
            print(value.mulit_line_str())
            print("are equal in measurement error range." if value == self.EQ_CONST else "are not equal in measurement error range.", end = "\n\n")


import setuptools

with open("README.md", "r") as readme_file:
    long_description = readme_file.read()

setuptools.setup(name="thad-roberts-model",
      version = '0.1.4',
      description = "Thad Roberts Model Library",
      long_description = long_description,
      long_description_content_type = "text/markdown",
      url = 'https://github.com/lukaszp/thad-roberts-model.git',
      author = 'Lukasz Paszke',
      author_email = 'lukasz.paszke@gmail.com',
      license = 'MIT',
      packages = setuptools.find_packages(),
      install_requires = [],
      extras_require = {'dev': ['twine']},
      classifiers = ["Programming Language :: Python :: 3",
                     "License :: OSI Approved :: MIT License",
                     "Operating System :: OS Independent"],
      python_requires = '>=3.8')


import math
from decimal import *

getcontext().prec = 28

class Measurement():
    '''
    Class for measurement with error and multiplier.
    It has API to work correctly in equations expressions in python. 
    '''
    def __init__(self, value, error, multiplier = "1"):
        self.value = Decimal(value) * Decimal(multiplier)
        self.absolute_error = Decimal(error) * Decimal(multiplier)
        assert self.absolute_error >= Decimal("0"), "Absolute error should be greater or equal to 0"
        if self.value == Decimal("0"): 
            self.relatvive_error = Decimal("0")
        else:
            #TODO: Fix division by 0 problem correctly, see tests
            self.relatvive_error = abs(self.absolute_error/self.value)
        
    def recalc_absolute(self):
        #TODO: Fix division by 0 problem correctly, see tests
        self.absolute_error = abs(self.relatvive_error * self.value)
    
    def recalc_relative(self):
        self.relatvive_error = abs(self.absolute_error/self.value)

        
    def __add__(self, other):
        self.value += other.value
        self.absolute_error += other.absolute_error
        self.recalc_relative()
        return self
    
    def __sub__(self, other):
        self.value -= other.value
        self.absolute_error += other.absolute_error
        self.recalc_relative()
        return self

    def __mul__(self, other):
        self.value *= other.value
        self.relatvive_error += other.relatvive_error
        self.recalc_absolute()
        return self
    
    def __truediv__(self, other):
        self.value = self.value/other.value
        self.relatvive_error += other.relatvive_error
        self.recalc_absolute()
        return self    
    
    def __pow__(self, other):
        self.value **= other.value
        self.relatvive_error *= abs(other.value)
        self.recalc_absolute()
        return self
    
    def __neg__(self):
        self.value = -self.value
        return self
        
    def __pos__(self):
        self.value = self.value
        return self
    
    def __eq__(self, other):
        if self.absolute_error != Decimal("0"):
            if (self.value + self.absolute_error >= other.value - other.absolute_error) and (other.value + other.absolute_error >= self.value - self.absolute_error):
                return True
            else: return False
        else:
            return self.value == other.value
    
    def __ne__(self, other):
        return bool(~(self == other))
        
    def __str__(self):
        if self.absolute_error == Decimal("0"):
            return "%.7f" % self.value
        return "[ %.7f, +/- %.7f, +/- %10.7f %%]" %(self.value, self.absolute_error, Decimal('100') * self.relatvive_error)
    def mulit_line_str(self):
        return "[    %.7f    ] = VALUE\n(+/- %.7f, +/- %9.7f %%) = SYMMETRIC ERROR" %(self.value, self.absolute_error, Decimal('100') * self.relatvive_error)

def v(value):
    '''Function returning constant value as an exact measurement in equations where it is necessary'''
    return Measurement(value, error = '0')
